use core::arch::asm;

use super::EFlags;

/// Disable interrupts
#[inline]
pub fn cli() {
	unsafe {
		asm!("cli", options(preserves_flags, nostack));
	}
}

/// Enable interrupts
#[inline]
pub fn sti() {
	unsafe {
		asm!("sti", options(preserves_flags, nostack));
	}
}

/// Run the given closure, disabling interrupts before running it (if they aren't already disabled).
/// Afterwards, interrupts are enabling again if they were enabled before.
///
/// This helps us prevent deadlocks, which can occur if
/// an interrupt handler tries to acquire a lock that was
/// locked at the time of the interrupt.
///
/// If you have other `enable` and `disable` calls _within_ the closure, things may not work as expected.
#[inline]
pub fn without_interrupts<F, R>(f: F) -> R
where
	F: FnOnce() -> R,
{
	// true if the interrupt flag is set (i.e. interrupts are enabled)
	let saved_intpt_flag = EFlags::read().contains(EFlags::INTERRUPT_FLAG);

	// if interrupts are enabled, disable them for now
	if saved_intpt_flag {
		cli();
	}

	let ret = f();

	if saved_intpt_flag {
		sti();
	}

	ret
}

/// Wraps the `in` instruction
pub unsafe fn inb(port: u32) -> u8 {
	let mut out;

	asm!(
		"in al, dx",
		out("al") out,
		in("dx") port,
	);

	return out;
}

/// Wraps the `out` instruction
pub unsafe fn outb(port: u32, value: u8) {
	asm!(
		"out dx, al",
		in("dx") port,
		in("al") value,
	);
}

/// Get the current value of the CS register
pub fn get_cs() -> u16 {
	let segment: u16;
	unsafe {
		asm!("mov {0:x}, cs", out(reg) segment, options(nomem, nostack, preserves_flags));
	}
	segment
}
